/**************************************************************************
Program to manipulate line terminators in ascii text files.
(Useful when exchanging files between different operating systems)
    Four options are available:
        1. Add a LF after each CR
        2. Add a CR before each LF
        3. Strip all LF characters.
        4. Strip all CR characters.

    Usage:
      LINETERM -tn (or /tn) <filename list> (wild cards OK)
        where n is the terminator option above (1,2,3 or 4) [1 is default]
      eg. Add a LF after each CR to all .FR files
           LINETERM -t1 *.fr

     Compiler:
        Compiled and tested with Turbo C 2.0 only

   by J.A. McKim - released to public domain May 10, 1990
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dir.h>
#include <dos.h>
#include <errno.h>

/* Define a few constants and BOOL data type */
#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef NULL
#define NULL 0
#endif
typedef int BOOL;

/* Define buffer size & max file matches per file name with wildcard */
#define  BUFSIZE  2048
#define  MAXNAME 100
#define TEMPFILE "LTERM.TMP"
long conv_term (FILE *,FILE *,long,short);
void main (int argc,char *argv[])
{
	short mode,iarg,ifile,nfile;
	long out_bytes,fsize[MAXNAME];
	struct ffblk fblk;
	FILE *fpin,*fpout;
	char fnames[MAXNAME][25]; /* holds wildcard expansions */

	if (argc < 3 || (argv[1][0] != '-' && argv[1][0] != '/') ||
	 tolower(argv[1][1]) != 't' ||
		 (mode = argv[1][2]-'0') < 1 || mode > 4) {
		printf (" Usage:");
		printf ("   LINETERM -tn (or /tn) <file list> (wild cards OK)\n");
		printf ("   where n is the terminator option:\n");
		printf ("       1. Add a LF after each CR\n");
		printf ("       2. Add a CR before each LF\n");
		printf ("       3. Strip all LF characters\n");
		printf ("       4. Strip all CR characters\n");
		printf (" Example:\n");
		printf ("   To place a LF after each CR in all TXT and DOC files use:\n");
		printf ("      LINETERM -t1 *.TXT *.DOC\n");
		exit(1);
	}
  	printf ("   File      bytes in  bytes out\n");
	for (iarg=2;iarg < argc;iarg++) {
		if (findfirst (argv[iarg],&fblk,0)) {  /* first name matching wild card */
			printf ("%s not found\n",argv[iarg]);
			continue;
		}
      /* Build array of file matches for next name in command line.
         This must be done before any mathes are converted because
         the findnext function sometimes returns the same name again
         since it is in a different directory location due to renaming
         the temporary file.
      */
      nfile = 0;     /* Number of file matches for next wildcard name */
      do {
	strcpy(fnames[nfile],fblk.ff_name);
	fsize[nfile++] = fblk.ff_fsize;
      } while (findnext(&fblk) == 0 && nfile < MAXNAME);
      if (nfile==MAXNAME)
         printf ("Warning: only the first %d files will be converted for wildcard file %s\n",
            nfile,argv[iarg]);
      for (ifile=0;ifile < nfile;ifile++) {
         printf ("%11s  %8ld  ",fnames[ifile],fsize[ifile]);
	 if ((fpin = fopen(fnames[ifile],"rb")) == NULL) {
		printf ("open error %d\n",errno);
		continue;
	 }
	 if ((fpout = fopen(TEMPFILE,"wb")) == NULL) {
		printf ("\Temp file open error %d\n",errno);
		exit(1);
	 }
	 out_bytes = conv_term (fpin,fpout,fsize[ifile],mode);
	 fclose (fpin);
	 fclose (fpout);
	 if (out_bytes > 0) {
		remove (fnames[ifile]);            /* delete input file */
		rename (TEMPFILE,fnames[ifile]);   /* rename temp file */
	 }
	 printf("%9ld\n",out_bytes);
      }
   }
   exit(0);
}
long conv_term (FILE *fpin, FILE *fpout,long num_bytes, short mode)
{
   /*  Read contents of input file with file pointer fpin, convert line
      terminator according to <mode> and write to file with file pointer
      <fpout>.  Argument <num_bytes> is the size of the input file.
   */
   char inbuf[BUFSIZE],outbuf[2*BUFSIZE];
   short in_bytes,i,j;
   long out_bytes;
   BOOL insert;
   char schar;
   insert = (mode < 3); /* insert a terminator for modes 1 and 2 */
   if (mode==1 || mode ==4)
      schar = '\r';  /* looking for carriage return */
   else
      schar = '\n';  /* otherwise looking for line feeds */
   out_bytes = 0;
   while (num_bytes > 0) {
      in_bytes = min(num_bytes,BUFSIZE);
      num_bytes -= in_bytes;
      if ((i=fread(inbuf,1,in_bytes,fpin)) != in_bytes) {
	 printf ("\nRead Error %d\n",errno);
         return(0);
      }
      for (i=0, j=0; i < in_bytes; i++) {
         if (inbuf[i] != schar)
            outbuf[j++] = inbuf[i];
         else {
            if (insert) {
               outbuf[j++] = '\r';  /* CR always first */
               outbuf[j++] = '\n';
            }
         }
      }
      fwrite (outbuf,j,1,fpout); /* write output buffer */
      out_bytes += j;   /* cumulative bytes written */
   }
   return (out_bytes);
}
